import WidgetRequiredValidator from '@/validator/widget-required-validator'
import WidgetRegexValidator from '@/validator/widget-regex-validator'

const debounce = window._.debounce
const XEventBus = window.APaaSSDK.context.XEventBus
const debounceWaitTime = 150
const AbilityControl = window.Vue.FormEngine.AbilityControl

const FormWidgetMixin = {
  data() {
    return {
      tabindex: '0',
      componentData: null,
      widgetRules: null,
      tip: '',
      valueChanged: false,
      validateDebounce: debounce((value) => {
        this.validate(value)
      }, 500),
      bsUnwatch: null,
      bsRefreshDebounce: debounce((newValue, oldValue) => {
        if (newValue !== oldValue) {
          // 调用触发业务事件
          console.log('调用值改变触发业务事件')
          let event
          // 在表格里
          if (this.widget.isInTable) {
            const key = this.$route.query.key
            event = {
              currentRowTableUuid: key.split('.')[0],
              currentRowIndex: parseInt(key.split('.')[1]),
              tableFormEngine: this.formEngine,
              vm: this,
              currentIndexRowId: this.tableData && this.tableData[this.tabindex].currentIndexRowId
            }
            try {
              this.sourceFormEngine &&
                this.sourceFormEngine.bsEventControl &&
                this.sourceFormEngine.bsEventControl.triggerEventValueChange(this.widget, event)
            } catch (error) {
              console.error(error)
            }
          } else {
            if (
              this.formEngine.engineContext.instance.documentId &&
              this.widget.desensitization &&
              this.formEngine.formDataControl.dataMaskingValue[this.widget.uuid] &&
              !this.formEngine.formDataControl.dataMaskingValue[this.widget.uuid].changed
            ) {
              this.formEngine.formDataControl.dataMaskingValue[this.widget.uuid].changed = true
            }
            try {
              this.formEngine.bsEventControl.triggerEventValueChange(this.widget, {
                vm: this
              })
            } catch (error) {
              console.error(error)
            }
          }
          if (
            this.formEngine.formDataControl.dataFilterComponentList.triggerComponents.includes(
              this.widget.uuid
            )
          ) {
            const dataSelectors = this.formEngine.formDataControl.dataFilterComponentList
              .dataSelectors
            Object.keys(dataSelectors).forEach((key) => {
              if (dataSelectors[key].includes(this.widget.uuid)) {
                XEventBus.emit(`REFRESH_SELECT_BOX`, {
                  uuid: key,
                  currentFormEngineKey: this.formEngine.engineContext.instance.instanceId
                })
              }
            })
          }
        }
      }, 500),
      regexValidatorText: '',
      specialComponents: [
        'FORM_DATA_STATISTICS',
        'FORM_SWITCH_SELECT',
        'FORM_MONEY_INPUT',
        'FORM_NUMBER_INPUT',
        'FORM_RADIO_INPUT',
        'FORM_DATEPICK_INPUT'
      ],
      debounceFormData: debounce(this.watchFormData, debounceWaitTime)
    }
  },
  props: {
    row: {
      type: Number
    },
    widget: {
      required: true
    },
    renderScene: {
      type: String,
      required: true,
      validator: function(value) {
        const values = ['ide', 'edit', 'read', 'widget']
        // 这个值必须匹配下列字符串中的一个
        return values.includes(value)
      }
    },
    propKey: {
      type: String,
      default: ''
    },

    validateKey: {
      type: String,
      default: ''
    },
    validateInfo: {
      type: Object
    },
    formData: {
      type: Object
    },
    globalFormData: {
      type: Object
    },

    globalData: {
      type: Object
    },

    formItemList: {
      type: Array,
      default: function() {
        return []
      }
    },

    valueValidatedStatus: {
      type: Boolean,
      default: true
    },
    sourceFormEngine: {
      type: Object,
      default: function() {
        return {}
      }
    },
    sonTableWidget: {
      type: Object,
      default: function() {
        return {}
      }
    }
  },
  inject: ['renderGlobal'],
  computed: {
    isShowButton() {
      /* eslint-disable */
      if (this.widget.isInTable) {
        return (
          this.formValue &&
          this.formEngine.engineContext.instance.documentId &&
          this.widget.showDesensitizationMark &&
          // eslint-disable-next-line standard/computed-property-even-spacing
          this.sourceFormEngine.formDataControl.dataMaskingValue[this.widget.tableUuid][
            this.formData.id
          ][this.widget.uuid] &&
          // eslint-disable-next-line standard/computed-property-even-spacing
          !this.sourceFormEngine.formDataControl.dataMaskingValue[this.widget.tableUuid][
            this.formData.id
          ][this.widget.uuid].changed
        )
      } else {
        return (
          this.formValue &&
          this.formEngine.engineContext.instance.documentId &&
          this.widget.showDesensitizationMark &&
          this.formEngine.formDataControl.dataMaskingValue[this.widget.uuid] &&
          !this.formEngine.formDataControl.dataMaskingValue[this.widget.uuid].changed
        )
      }
    },
    formValue: {
      get() {
        // 回调函数 当需要读取当前属性值是执行，根据相关数据计算并返回当前属性的值
        this.valueChanged = false
        if (this.valueValidatedStatus) {
          return this.propKey ? this.formData[this.propKey] : undefined
        } else {
          return undefined
        }
      },
      set(value) {
        if (this.formData[this.propKey] !== value) {
          // 避免tab切换引起的校验
          this.valueChanged = true
          // this.formData[this.propKey] = value
          this.$set(this.formData, this.propKey, value)
          // this.$emit('validate:propKey', this.propKey)
          // this.formEngine.formDataControl.updateFormValue({ ...this.formData })
          // this.$emit('update:formData', { ...this.formData }, this.propKey)
          if (!this.widget.isInTable) {
            this.$formEventEmit('change', value)
          }
          // 校验
          if (this.noDebounceValidate) {
            this.validate(value)
          } else {
            this.validateDebounce(value)
          }
          // 数据统计组件、开关组件
          if (!this.specialComponents.includes(this.widget.componentType) && this.formEngine) {
            this.formEngine.formDataControl.ctlFormDataChanged = true
          }
        }
      }
    },
    formEngine() {
      if (typeof this.renderGlobal === 'function') {
        return this.renderGlobal()
      }
      return this.renderGlobal
    },
    validatorRules() {
      let rules = []
      if (this.renderScene === 'edit') {
        if (this.showRequired) {
          if (this._validate) {
            rules.push(
              this._validate(
                'required',
                `${this.widget && this.widget.label}${this.$t('mFormValidator.required')}`
              )
            )
          }
        }
        if (this.widget.validatorStatus) {
          if (this._validate) {
            rules.push(
              this._validate(
                WidgetRegexValidator(
                  `${this.regexValidatorText}`,
                  this.widget.validatorList[0].validatorMessage
                )
              )
            )
          }
        }
        if (this.widgetRules) {
          rules = [...rules, ...this.widgetRules]
        }
      }
      return rules
    },
    showRequired() {
      // return this.widget.required && !this.widget.readOnly && !this.widget.hidden
      // 这里需要注意，隐藏后，相应的dom直接没了，就没有所隐藏的widget，所以隐藏后showRequired不会发生变化，不会触发watch
      // 但是当只读 readonly发生改变后 是能够触发watch的
      return this.widget && this.widget.required && !this.widget.readOnly
    },
    disabled() {
      return (
        this.renderScene === 'read' ||
        this.renderScene === 'ide' ||
        (this.widget && (this.widget.enableRule || this.widget.readOnly))
      )
    },
    idePlaceholder() {
      if (this.widget.defaultValueType === 'rule') {
        return this.$t('mFormWidget.formulaRule')
      } else {
        return this.widget.enableRule
          ? '自动生成'
          : this.widget.readOnly
          ? ''
          : this.widget.placeholder
      }
    },
    placeholder() {
      return this.widget.enableRule
        ? '自动生成'
        : this.widget.readOnly
        ? ''
        : this.widget.placeholder
    },
    proxyFormItemName() {
      const componentPrefix =
        this.$_installMobileOptions && this.$_installMobileOptions.componentPrefix
      const name = 'XProxyFormItem'
      if (componentPrefix) {
        return `${componentPrefix}${name}`
      } else {
        return name
      }
    },
    xEmptyName() {
      const componentPrefix =
        this.$_installMobileOptions && this.$_installMobileOptions.componentPrefix
      const name = 'XEmpty'
      if (componentPrefix) {
        return `${componentPrefix}${name}`
      } else {
        return name
      }
    },
    xFormListCardName() {
      const componentPrefix =
        this.$_installMobileOptions && this.$_installMobileOptions.componentPrefix
      const name = 'XFormListCard'
      if (componentPrefix) {
        return `${componentPrefix}${name}`
      } else {
        return name
      }
    },
    xResizeableName() {
      const componentPrefix =
        this.$_installMobileOptions && this.$_installMobileOptions.componentPrefix
      const name = 'XResizeable'
      if (componentPrefix) {
        return `${componentPrefix}${name}`
      } else {
        return name
      }
    },
    xTagName() {
      const componentPrefix =
        this.$_installMobileOptions && this.$_installMobileOptions.componentPrefix
      const name = 'XTag'
      if (componentPrefix) {
        return `${componentPrefix}${name}`
      } else {
        return name
      }
    },
    xTagGroupName() {
      const componentPrefix =
        this.$_installMobileOptions && this.$_installMobileOptions.componentPrefix
      const name = 'XTagGroup'
      if (componentPrefix) {
        return `${componentPrefix}${name}`
      } else {
        return name
      }
    },
    XUserTagName() {
      const componentPrefix =
        this.$_installMobileOptions && this.$_installMobileOptions.componentPrefix
      const name = 'XUserTag'
      if (componentPrefix) {
        return `${componentPrefix}${name}`
      } else {
        return name
      }
    },
    xInputMoneyName() {
      const componentPrefix =
        this.$_installMobileOptions && this.$_installMobileOptions.componentPrefix
      const name = 'XInputMoney'
      if (componentPrefix) {
        return `${componentPrefix}${name}`
      } else {
        return name
      }
    },
    xInputPhoneName() {
      const componentPrefix =
        this.$_installMobileOptions && this.$_installMobileOptions.componentPrefix
      const name = 'XInputPhone'
      if (componentPrefix) {
        return `${componentPrefix}${name}`
      } else {
        return name
      }
    },
    xAreaName() {
      const componentPrefix =
        this.$_installMobileOptions && this.$_installMobileOptions.componentPrefix
      const name = 'FormSelectRegionDrawer'
      if (componentPrefix) {
        return `${componentPrefix}${name}`
      } else {
        return name
      }
    },
    xUploadName() {
      const componentPrefix =
        this.$_installMobileOptions && this.$_installMobileOptions.componentPrefix
      const name = 'XUpload'
      if (componentPrefix) {
        return `${componentPrefix}${name}`
      } else {
        return name
      }
    },
    mobileFormSettings() {
      return {
        widgetStyle: this.widget.mobileWidgetStyle || {},
        border: this.widget.mobileBorder || {}
      }
    },
    // 子表组件的uuid数组
    tableUuid() {
      const formComponents = this.renderGlobal.formDataControl.allTileFormItemList || []
      return formComponents
        .filter((comp) => comp.componentType === 'FORM_WIDGET_SON_TABLE')
        .map((item) => item.uuid)
    },
    // 非子表数据
    formDataWithoutTableData() {
      const formData = { ...this.formData }
      this.tableUuid.forEach((uuid) => {
        delete formData[uuid]
      })
      const assoValue = formData
      let jsonObject = ''
      try {
        jsonObject = JSON.stringify(assoValue)
      } catch (error) {
        jsonObject = assoValue
      }
      return jsonObject
    }
  },
  watch: {
    showRequired: {
      handler(newVal, oldValue) {
        if (oldValue && !newVal) {
          this.$nextTick(() => {
            this.validate(newVal)
          })
        }
      }
    },
    formDataWithoutTableData: {
      handler(newValue, oldValue) {
        if (newValue) {
          if (
            this.widget.validatorList &&
            this.widget.validatorList[0].validatorConfig.length !== 0
          ) {
            this.regexValidatorText = this.titleDesArrToText(
              newValue,
              this.widget.validatorList[0].validatorConfig,
              AbilityControl.FORM_VALIDATOR_FORM_FIELD
            )
          }
          if (this.widget.componentType === 'FORM_WIDGET_SON_TABLE') {
            this.tableValidate(newValue)
          }
        }
        this.debounceFormData(this.formData)
      },
      immediate: true,
      deep: true
    }
  },
  created() {
    if (!this.widget.extendTempObj) {
      this.$set(this.widget, 'extendTempObj', {})
    }
    if (this.formEngine && this.formEngine.formDataControl) {
      const currentWidget = this.formEngine.formDataControl.formConfig.detailPage.formComponents.find(
        (comp) => {
          return comp.uuid === this.widget.uuid
        }
      )
      currentWidget && this.$set(currentWidget, 'validate', this.validate)
    }
    const formItemWidget = this.formItemList.find((item) => item.uuid === this.widget.uuid)
    if (formItemWidget) {
      this.$set(formItemWidget, 'validate', this.validate)
    }
  },
  mounted() {
    // mixin的beforeCreate > 父beforeCreate > mixin的created > 父created > mixin的beforeMount > 父beforeMount > 子beforeCreate > 子created > 子beforeMount > 子mounted > mixin的mounted >父mounted
    // 刷新表格
    if (this.renderScene === 'edit' || this.renderScene === 'read') {
      const timer = setTimeout(() => {
        clearTimeout(timer)
        this.addBsUnwatch()
      }, 0)
    }
  },
  beforeDestroy() {
    this.debounceFormData.cancel()
  },
  destroyed() {
    if (this.bsUnwatch) {
      this.bsUnwatch()
    }
  },

  methods: {
    watchFormData(newValue, oldValue) {
      if (newValue) {
        let titleDescription = ''
        if (
          this.widget.titleDescription &&
          (!this.widget.titleDescriptionOptions || this.widget.titleDescriptionOptions.length === 0)
        ) {
          titleDescription = this.widget.titleDescription
        } else {
          titleDescription = this.titleDesArrToText(
            newValue,
            this.widget.titleDescriptionOptions,
            AbilityControl.TITLE_DESCRIPTION_FORM_FIELD
          )
        }
        if (
          this.widget.validatorList &&
          this.widget.validatorList[0].validatorConfig.length !== 0
        ) {
          this.regexValidatorText = this.titleDesArrToText(
            newValue,
            this.widget.validatorList[0].validatorConfig,
            AbilityControl.FORM_VALIDATOR_FORM_FIELD
          )
        }
        // if (this.widget.componentType === 'FORM_WIDGET_SON_TABLE') {
        //   if (Array.isArray(this.widget.children) && this.widget.children.length > 0) {
        //     this.widget.children.forEach((colItem) => {
        //       let colTitleDescription = ''
        //       if (
        //         colItem.titleDescription &&
        //         (!colItem.titleDescriptionOptions || colItem.titleDescriptionOptions.length === 0)
        //       ) {
        //         colTitleDescription = colItem.titleDescription
        //       } else {
        //         colTitleDescription = this.titleDesArrToText(
        //           newValue,
        //           colItem.titleDescriptionOptions
        //         )
        //       }
        //       this.$set(colItem, 'titleDescription', colTitleDescription)
        //     })
        //   }
        // }
        this.tip = titleDescription
      }
    },
    addBsUnwatch() {
      this.bsUnwatch = this.$watch(
        function() {
          const assoValue = this.formValue
          let jsonObject = assoValue
          // 如果本身是string类型不执行以下逻辑
          if (typeof assoValue !== 'string') {
            try {
              jsonObject = JSON.stringify(assoValue)
            } catch (error) {
              jsonObject = assoValue
            }
          }
          return jsonObject
        },
        (newValue, oldValue) => {
          // 转为string类型判断，避免同样的值，类型不同，导致触发两次
          if (String(newValue) !== String(oldValue)) {
            this.validateDebounce()
            if (!this.widget.isInTable) {
              this.bsRefreshDebounce(newValue, oldValue)
            }
          }
        }
      )
    },
    _validate(type, message, trigger = ['blur', 'change'], isI18n = false) {
      // 生成验证器
      const validator = {
        trigger: trigger
      }
      if (typeof type === 'string') {
        validator.type = type
        if (type === 'required') {
          validator.validator = WidgetRequiredValidator(isI18n ? this.$t(message) : message)
        }
        validator.message = isI18n ? this.$t(message) : message
      } else if (typeof type === 'function') {
        validator.validator = type
      }

      return validator
    },
    updatePropValue(key, value) {
      // 判断有这个组件时才进行赋值，如果不判断且如果key不存的话则会生成一个不存在的组件值
      if (
        this.formData.hasOwnProperty(key) ||
        (this.formEngine && this.formEngine.formDataControl.ctlComponentMap.has(key))
      ) {
        this.formData[key] = value
        this.$set(this.formData, key, value)
        this.$nextTick(() => {
          this.$emit('formEventEmit', {
            eventName: 'change',
            event: value,
            propKey: key,
            widget: this.formEngine.formDataControl.ctlComponentMap.get(key)
          })
        })
      }
      // this.formEngine.formDataControl.updateFormValue({ ...this.formData })
      // this.$emit('update:formData', { ...this.formData }, this.key)
    },
    mapAdapter(...args) {
      return this.$formBuildEngine.mapAdapter(args)
    },
    mapEvent(...args) {
      return this.$formBuildEngine.mapEvent(args)
    },
    $formEventEmit(eventName, event) {
      this.$emit(eventName, event)
      this.$emit('formEventEmit', {
        eventName: eventName,
        propKey: this.propKey,
        event: event,
        widget: this.widget
      })
    },
    validate(value) {
      if (!this.validatorRules || this.validatorRules.length === 0) {
        const validateInfo = { ...this.validateInfo, validate: 'validate' }
        this.$set(this.widget.extendTempObj, 'validateInfo', validateInfo)
        this.$emit('validateChanged', validateInfo)
        return
      }
      let flag = false
      for (const rule of this.validatorRules) {
        if (flag) {
          break
        }
        rule.validator(
          rule,
          value || this.formValue,
          (e) => {
            if (e) {
              flag = true
              const validateInfo = {
                ...this.validateInfo,
                validate: 'error',
                msg: e.message,
                uuid: this.validateKey
              }
              this.$set(this.widget.extendTempObj, 'validateInfo', validateInfo)
              this.$emit('validateChanged', validateInfo)
            } else {
              const validateInfo = {
                ...this.validateInfo,
                validate: 'validate',
                uuid: this.validateKey
              }
              this.$set(this.widget.extendTempObj, 'validateInfo', validateInfo)
              this.$emit('validateChanged', validateInfo)
            }
          },
          this.widget
        )
      }
    },
    isTableMaskingData() {
      /* eslint-disable */
      if (!this.widget.isInTable) {
        return true
      } else if (!this.formData.id) {
        return false
      } else {
        return (
          this.sourceFormEngine.formDataControl.dataMaskingValue[this.widget.tableUuid] &&
          this.sourceFormEngine.formDataControl.dataMaskingValue[this.widget.tableUuid][
            this.formData.id
          ] &&
          this.sourceFormEngine.formDataControl.dataMaskingValue[this.widget.tableUuid][
            this.formData.id
          ][this.widget.uuid] &&
          !this.sourceFormEngine.formDataControl.dataMaskingValue[this.widget.tableUuid][
            this.formData.id
          ][this.widget.uuid].changed
        )
      }
      /* eslint-disable */
    },
    showRealValue() {
      const { documentId, formId } = this.globalData
      const { uuid, isInTable, tableUuid } = this.widget
      const tableDocumentId = this.formData.documentId
      const key = this.widget.modelField.split('.')[1]
      this.formEngine.actionControl
        .executeActionWithPromise('GET_FORM_REAL_DATA_ACTION', {
          formId,
          documentId,
          uuid,
          isInTable,
          tableUuid,
          tableDocumentId
        })
        .then((resp) => {
          // 设置值
          resp && this.$set(this, 'formValue', resp[key])
          // 修改显示状态
          const componentConfig = this.formEngine.formDataControl.componentMap.get(uuid)
          componentConfig.showDesensitizationMark = false
          // 修改值的状态
          let dataMaskingValue
          if (
            isInTable &&
            this.formEngine.formDataControl.dataMaskingValue[this.widget.tableUuid] &&
            this.formEngine.formDataControl.dataMaskingValue[this.widget.tableUuid][
              this.formData.id
            ]
          ) {
            dataMaskingValue = this.formEngine.formDataControl.dataMaskingValue[
              this.widget.tableUuid
            ][this.formData.id][this.widget.uuid]
          } else {
            dataMaskingValue = this.formEngine.formDataControl.dataMaskingValue[this.widget.uuid]
          }
          dataMaskingValue.changed = true
        })
    },
    setFormDataChanged(value) {
      this.formEngine.formDataControl.ctlFormDataChanged = value || false
    },
    titleDesArrToText(formData, arr, abilityCode) {
      let text = ''
      arr &&
        arr.forEach((item) => {
          if (item.type === 'TEXT') {
            text += item.value.replace(/\n/g, '')
          } else if (item.type === 'COMP') {
            const allComps = this.formEngine.formDataControl.allTileFormItemList
            const compConfig = allComps && allComps.find((_item) => _item.uuid === item.value)
            const textValue =
              AbilityControl.formatFiledValue({
                fieldType: compConfig && compConfig.componentType,
                value: formData[item.value],
                fieldConfig: compConfig,
                fieldId: item.value,
                abilityCode: abilityCode
              }) || ''
            text += textValue
            // text += textValue
            // if (
            //   ['FORM_RADIO_INPUT', 'FORM_SELECT_INPUT', 'FORM_CHECKBOX_INPUT'].includes(
            //     compConfig.componentType
            //   )
            // ) {
            //   let selectData = [...compConfig.chooseOptions].filter((_item) => {
            //     return formData[item.value] && formData[item.value].includes(_item.id)
            //   })
            //   text += getTextFromFormValue(selectData, compConfig)
            // } else {
            //   text += getTextFromFormValue(formData[item.value], compConfig)
            // }
          } else if (item.type === 'RULE') {
            const ruleConfig = JSON.parse(item.detailRuleConfig)
            let rule = {
              ruleId: ruleConfig.advancedRuleId,
              ruleStack: ruleConfig.advancedRuleConfig.advancedRuleList
            }
            let result
            try {
              result = this.formEngine.ruleControl.ruleEngine.executeRule(rule)
            } catch (err) {
              result = ''
            }
            text += result
          } else if (item.type === 'URL') {
            let url = this.titleDesArrToText(formData, JSON.parse(item.urlConfig), abilityCode)
            let hrefUrl = url
            let reg = new RegExp('^(http|https):\\/\\/(?!.*\\s)(?!.*\\.\\.)[^\\s]+$')
            if (!reg.test(url)) {
              hrefUrl = 'about:blank#blocked'
            }
            let urlText = item.hasShowText === 'true' ? item.name : url
            text += `<a target="${
              item.newWindowStatus === 'true' ? '_blank' : '_self'
            }" data-target="${
              item.newWindowStatus === 'true' ? '_blank' : '_self'
            }" class="title-description-url" href="${hrefUrl}">${urlText}</a>`
          }
        })
      text += ''
      return text
    },
    /**
     * 用于拆分子表内sourceFormEngine中formValue数据，该数据中存在当前行数据和主表数据平铺
     * @param {*} sourceFormEngine 主表引擎
     * @param {*} formEngine 子表引擎
     * @param {*} tableUuid 当前子表uuid
     */
    splitFormValueHandle(sourceFormEngine, formEngine, tableUuid) {
      if (!sourceFormEngine || !sourceFormEngine.formDataControl || !tableUuid) {
        return {
          formDetailData: formEngine.formDataControl.formValue,
          currentRowTableData: formEngine.formDataControl.currentRowTableData
        }
      }
      // 获取子表组件uuid列表
      const sonTableCompList = formEngine.formDataControl.formItemList.map((item) => item.uuid)
      const { formValue: tileFormValue } = sourceFormEngine.formDataControl
      // a为过滤的key，b为拆分的对象
      // 拆分主表数据和平铺的子表当前行数据
      const classifyFormValue = function(b, a) {
        const formDetailData = {}

        // 遍历对象a的键
        for (const key in a) {
          if (!b.includes(key)) {
            // 如果键不存在于数组b中，将其添加到formDetailData对象中
            formDetailData[key] = a[key]
          }
        }
        return formDetailData
      }
      // 获取有效的当前行子表数据
      const generateCurrentRowTableData = function(tableUuid, formValue) {
        const currentRowTableData = {}
        for (const key in formValue) {
          if (
            formValue.hasOwnProperty(key) &&
            formValue[key] !== null &&
            formValue[key] !== undefined
          ) {
            currentRowTableData[key] = formValue[key]
          }
        }
        return { [tableUuid]: [currentRowTableData] }
      }
      return {
        currentRowTableData: generateCurrentRowTableData(
          tableUuid,
          formEngine.formDataControl.formValue
        ),
        formDetailData: classifyFormValue(sonTableCompList, tileFormValue)
      }
    }
  }
}

export default FormWidgetMixin
